@@ -1,6 +1,6 @@
 // TimerPreferencesPanel.cc --- Preferences widgets for a timer
 //
-// Copyright (C) 2002 - 2011 Raymond Penners <raymond@dotsphinx.com>
+// Copyright (C) 2002 - 2013 Raymond Penners <raymond@dotsphinx.com>
 // All rights reserved.
 //
 // This program is free software: you can redistribute it and/or modify
@@ -53,10 +53,17 @@ TimerPreferencesPanel::TimerPreferencesPanel
 (BreakId t,
  Glib::RefPtr<Gtk::SizeGroup> hsize_group,
  Glib::RefPtr<Gtk::SizeGroup> vsize_group)
-  : Gtk::VBox(false, 6),
-         max_prelude_adjustment(0, 1, 100)
-#ifdef HAVE_EXERCISES
+  : Gtk::VBox(false, 6)
+#ifdef HAVE_GTK3
+  ,max_prelude_adjustment(Gtk::Adjustment::create(0, 1, 100))
+# ifdef HAVE_EXERCISES
+  ,exercises_adjustment(Gtk::Adjustment::create(0, 0, 10))
+# endif
+#else
+  ,max_prelude_adjustment(0, 1, 100)
+# ifdef HAVE_EXERCISES
   ,exercises_adjustment(0, 0, 10)
+# endif
 #endif
 {
   connector = new DataConnector();
@@ -71,7 +78,7 @@ TimerPreferencesPanel::TimerPreferencesPanel
   enabled_cb->signal_toggled().connect(sigc::mem_fun(*this, &TimerPreferencesPanel::on_enabled_toggled));
 
 
-  HigCategoriesPanel *categories = Gtk::manage(new HigCategoriesPanel());;
+  HigCategoriesPanel *categories = Gtk::manage(new HigCategoriesPanel());
 
   Gtk::Widget *prelude_frame = Gtk::manage(create_prelude_panel());
   Gtk::Widget *timers_frame = Gtk::manage(create_timers_panel
@@ -91,7 +98,7 @@ TimerPreferencesPanel::TimerPreferencesPanel
   pack_start(*box, false, false, 0);
 
   connector->connect(CoreConfig::CFG_KEY_BREAK_ENABLED % break_id, dc::wrap(enabled_cb));
-  
+
   set_border_width(12);
 }
 
@@ -109,19 +116,19 @@ TimerPreferencesPanel::create_prelude_panel()
   HigCategoryPanel *hig = Gtk::manage(new HigCategoryPanel(_("Break prompting")));
 
   prelude_cb = Gtk::manage(new Gtk::CheckButton(_("Prompt before breaking")));
-  hig->add(*prelude_cb);
+  hig->add_widget(*prelude_cb);
 
   Gtk::HBox *max_box = Gtk::manage(new Gtk::HBox());
   has_max_prelude_cb = Gtk::manage(new Gtk::CheckButton(_("Maximum number of prompts:")));
   max_prelude_spin = Gtk::manage(new Gtk::SpinButton(max_prelude_adjustment));
   max_box->pack_start(*has_max_prelude_cb, false, false, 0);
   max_box->pack_start(*max_prelude_spin, false, false, 0);
-  hig->add(*max_box);
+  hig->add_widget(*max_box);
 
   connector->connect(CoreConfig::CFG_KEY_BREAK_MAX_PRELUDES % break_id,
                      dc::wrap(prelude_cb),
                      sigc::mem_fun(*this, &TimerPreferencesPanel::on_preludes_changed));
-  
+
   connector->connect(CoreConfig::CFG_KEY_BREAK_MAX_PRELUDES % break_id,
                      dc::wrap(has_max_prelude_cb),
                      sigc::mem_fun(*this, &TimerPreferencesPanel::on_preludes_changed),
@@ -141,16 +148,21 @@ TimerPreferencesPanel::create_options_panel()
 
   // Ignorable
   ignorable_cb = Gtk::manage(new Gtk::CheckButton
-                        (_("Show 'Postpone' and 'Skip' button")));
-  hig->add(*ignorable_cb);
+                             (_("Show 'Postpone' button")));
+  hig->add_widget(*ignorable_cb);
+
+  // Skippable
+  skippable_cb = Gtk::manage(new Gtk::CheckButton
+                             (_("Show 'Skip' button")));
+  hig->add_widget(*skippable_cb);
 
   // Sensitive for activity
   activity_sensitive_cb = Gtk::manage(new Gtk::CheckButton
-                        (_("Suspend timer when inactive")));
+                                      (_("Suspend timer when inactive")));
 #if REMOVED_IN_FAVOR_OF_READING_MODE__REMOVE_LATER
   hig->add(*activity_sensitive_cb);
 #endif
-  
+
   // Break specific options
 #ifdef HAVE_EXERCISES
   exercises_spin = NULL;
@@ -162,7 +174,7 @@ TimerPreferencesPanel::create_options_panel()
     {
       monitor_cb
         = Gtk::manage(new Gtk::CheckButton(_("Regard micro-breaks as activity")));
-      hig->add(*monitor_cb);
+      hig->add_widget(*monitor_cb);
     }
 #endif
 
@@ -170,14 +182,14 @@ TimerPreferencesPanel::create_options_panel()
   if (break_id == BREAK_ID_REST_BREAK)
     {
       exercises_spin = Gtk::manage(new Gtk::SpinButton(exercises_adjustment));
-      hig->add(_("Number of exercises:"), *exercises_spin);
+      hig->add_label(_("Number of exercises:"), *exercises_spin);
     }
 #endif
   if (break_id == BREAK_ID_REST_BREAK)
     {
-      auto_natural_cb = Gtk::manage(new Gtk::CheckButton(_("Start natural restbreak when screen is locked")));
-      hig->add(*auto_natural_cb);
-      
+      auto_natural_cb = Gtk::manage(new Gtk::CheckButton(_("Start restbreak when screen is locked")));
+      hig->add_widget(*auto_natural_cb);
+
       connector->connect(GUIConfig::CFG_KEY_BREAK_AUTO_NATURAL % break_id,
                          dc::wrap(auto_natural_cb));
     }
@@ -188,7 +200,10 @@ TimerPreferencesPanel::create_options_panel()
   connector->connect(GUIConfig::CFG_KEY_BREAK_IGNORABLE % break_id,
                      dc::wrap(ignorable_cb));
 
-  
+  connector->connect(GUIConfig::CFG_KEY_BREAK_SKIPPABLE % break_id,
+                     dc::wrap(skippable_cb));
+
+
 #ifdef HAVE_EXERCISES
   if (break_id == BREAK_ID_REST_BREAK)
     {
@@ -200,6 +215,7 @@ TimerPreferencesPanel::create_options_panel()
   connector->connect(CoreConfig::CFG_KEY_TIMER_MONITOR % break_id,
                      dc::wrap(monitor_cb),
                      sigc::mem_fun(*this, &TimerPreferencesPanel::on_monitor_changed));
+
   return hig;
 }
 
@@ -212,7 +228,7 @@ TimerPreferencesPanel::create_timers_panel
 
   // Limit time
   limit_tim = Gtk::manage(new TimeEntry());
-  Gtk::Label *limit_lab = hig->add(break_id == BREAK_ID_DAILY_LIMIT
+  Gtk::Label *limit_lab = hig->add_label(break_id == BREAK_ID_DAILY_LIMIT
            ? _("Time before end:")
            : _("Time between breaks:"), *limit_tim);
   hsize_group->add_widget(*limit_lab);
@@ -227,7 +243,7 @@ TimerPreferencesPanel::create_timers_panel
 
       Gtk::Label *auto_reset_lab = Gtk::manage(new Gtk::Label(auto_reset_txt));
       hsize_group->add_widget(*auto_reset_lab);
-      hig->add(*auto_reset_lab, *auto_reset_tim);
+      hig->add_label(*auto_reset_lab, *auto_reset_tim);
     }
   else
     {
@@ -236,7 +252,7 @@ TimerPreferencesPanel::create_timers_panel
 
   // Snooze time
   snooze_tim = Gtk::manage(new TimeEntry());
-  Gtk::Label *snooze_lab = hig->add(_("Postpone time:"), *snooze_tim);
+  Gtk::Label *snooze_lab = hig->add_label(_("Postpone time:"), *snooze_tim);
   hsize_group->add_widget(*snooze_lab);
 
   vsize_group->add_widget(*hig);
@@ -268,7 +284,7 @@ TimerPreferencesPanel::on_preludes_changed(const std::string &key, bool write)
     return true;
 
   inside = true;
-  
+
   IConfigurator *config = CoreFactory::get_configurator();
   if (write)
     {
@@ -277,7 +293,11 @@ TimerPreferencesPanel::on_preludes_changed(const std::string &key, bool write)
         {
           if (has_max_prelude_cb->get_active())
             {
+#ifdef HAVE_GTK3
+              mp = (int) max_prelude_adjustment->get_value();
+#else
               mp = (int) max_prelude_adjustment.get_value();
+#endif
             }
           else
             {
@@ -311,7 +331,11 @@ TimerPreferencesPanel::on_preludes_changed(const std::string &key, bool write)
             {
               prelude_cb->set_active(true);
               has_max_prelude_cb->set_active(true);
+#ifdef HAVE_GTK3
+              max_prelude_adjustment->set_value(value);
+#else
               max_prelude_adjustment.set_value(value);
+#endif
             }
 
           set_prelude_sensitivity();
@@ -319,7 +343,7 @@ TimerPreferencesPanel::on_preludes_changed(const std::string &key, bool write)
     }
 
   inside = false;
-  
+
   return true;
 }
 
@@ -347,7 +371,7 @@ TimerPreferencesPanel::on_monitor_changed(const string &key, bool write)
     {
       string monitor_name;
       bool ok = config->get_value(key, monitor_name);
-      if (ok)
+      if (ok && monitor_name != "")
         {
           bool s = monitor_cb->is_sensitive();
           monitor_cb->set_active(monitor_name != "");
@@ -375,6 +399,7 @@ TimerPreferencesPanel::enable_buttons()
   bool on = enabled_cb->get_active();
 
   ignorable_cb->set_sensitive(on);
+  skippable_cb->set_sensitive(on);
   activity_sensitive_cb->set_sensitive(on);
 
 #ifdef HAVE_MICRO_BREAK_ACTIVITY
